# --8<--8<--8<--8<--
#
# Copyright (C) 2010 Smithsonian Astrophysical Observatory
#
# This file is part of make-world
#
# make-world is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# -->8-->8-->8-->8--

package MakeWorld::Check;

use strict;
use warnings;

use Carp;

use Params::Check qw[];
use List::Util;

use base 'Exporter';

use Storable qw[ dclone ];

our @EXPORT_OK =
  qw[
	check
	%BOOLEAN
	bool2val
   ];


my %bool = ( yes => [ qw[ yes 1 on ] ],
	     no  => [ qw[ no 0 off ] ],
	   );

my %bool2val = ( ( map { $_ => 1 } @{ $bool{yes} } ),
		 ( map { $_ => 0 } @{ $bool{no}  } ),
	   );


our %BOOLEAN =
  (
   allow => sub { exists $bool2val{lc $_[0]} },
   xfrm  => sub { $_[0] = $bool2val{$_[0]} }
  );


sub check {

    my ( $tpl, $attr ) = @_;

    # don't pass xfrm attributes along to Params::Check
    # Storable::dclone doesn't do what's needed here for code references (e.g.,
    # just pass them along), so delete xfrm and reinsert later.
    my %ntpl;

    $ntpl{$_}{xfrm} = delete $tpl->{$_}{xfrm}
      foreach grep { exists $tpl->{$_}{xfrm} } keys %$tpl;

    # Params::Check isn't quite consistent in how it handles errors
    # so have to do some extra work
    my $nattr;
    {
	local $Params::Check::ONLY_ALLOW_DEFINED = 1;
	local $Params::Check::PRESERVE_CASE = 1;
	local $Params::Check::VERBOSE = 0;
	local $Params::Check::WARNINGS_FATAL = 1;
	$nattr = eval { Params::Check::check( $tpl, $attr ) };

	if ( ! $nattr )
	{
	  croak( 'internal error: $tpl or $attr not set' )
	      if ! $tpl or ! $attr;

	  my $error = Params::Check::last_error();
	  $error =~ s/\s+for.*$//gm;
	  $error =~ s/key/attribute/ig;
	  die( $error . "\n" );
	}

    }

    $tpl->{$_}{xfrm} = delete $ntpl{$_}{xfrm}
      foreach grep { exists $ntpl{$_}{xfrm} } keys %ntpl;

    # transformations
    while( my ( $key, $val ) = each %$nattr )
    {
	$tpl->{$key}->{xfrm}->($nattr->{$key})
	  if exists $tpl->{$key} && exists $tpl->{$key}->{xfrm}
    }

    return $nattr;
}
